#formtools-form-body {
    max-width: 700px;
}

div .formtools-content {
    min-height: 400px;
}

.formtools-form-pages .badge {
    background-color: #4868b3;
}

.formtools-form-pages .remove {
    position: absolute;
    right: 2.5em;
    color: #5C6273;
}

.formtools-component-menu .remove {
    display: none;
}

.formtools-form-pages .edit {
    position: absolute;
    right: 4em;
    color: #5C6273;
}

.formtools-component-menu .edit {
    display: none;
}

.formtools-component-menu .badge {
    display: none;
}

.formtools-element-placeholder {
    border: 2px dotted #ffc107 !important;
    display: none;
}

.formtools-element-placeholder.active {
    display: block;
}

.formtools-element.current p {
    opacity: 0.7;
}

.formtools-form-graphic-box {
    background-color: #EEEEFF;
}

p.formtools-form-graphic-text {
    font-weight: bold;
    font-size: 1.5rem;
}

div .formtools-form-list {
    max-width: 85%;
}

div .formtools-admin-description {
    max-width: 1000px;
}

.formtools-form-text > p {
    color: black;
    line-height: 1.5;
}

#formtools-edit .card-body {
    padding: 15px 15px 15px 15px;
}

.formtools-form-pages input::placeholder {
    opacity: 0.5;
}

.formtools-element-empty-room {
    height: 200px;
}

#formtools-edit .custom-control-label.has-text-input::before,
#formtools-edit .custom-control-label.has-text-input::after {
  top: 0.5rem;
}

/* Make select options fully clickable */
#formtools-edit .custom-checkbox {
    z-index: auto;
}
